#!/usr/bin/env python

"""
  Get transformations with a given Status
"""

__RCSID__ = "$Id$"

import DIRAC
from DIRAC.Core.Base.Script import Script
from DIRAC.TransformationSystem.Client.TransformationClient import TransformationClient


@Script()
def main():
    Script.registerSwitch("", "Status=", "status")
    Script.parseCommandLine()

    status = None

    for switch in Script.getUnprocessedSwitches():
        if switch[0] == "Status":
            status = switch[1].capitalize()

    tc = TransformationClient()
    res = tc.getTransformationWithStatus(status)

    if not res["OK"]:
        DIRAC.gLogger.error(res["Message"])
        DIRAC.exit(-1)

    transIDs = res["Value"]

    if transIDs:
        DIRAC.gLogger.notice(
            f"Found {len(transIDs)} transformations with status {status}"
        )
    else:
        DIRAC.gLogger.notice("No transformations selected with status", status)
        DIRAC.exit(0)

    strTransIDs = []
    for transID in transIDs:
        strTransIDs.append(str(transID))

    DIRAC.gLogger.notice(f"{','.join(strTransIDs)}")

    DIRAC.exit(0)


if __name__ == "__main__":
    main()



#!/usr/bin/env python

"""
Remove files downloaded as job InputData having a Bad checksum
"""

__RCSID__ = "$Id$"

# Generic imports
import os

# DIRAC imports
import DIRAC
from DIRAC.Core.Base.Script import Script

Script.parseCommandLine()

# Specific DIRAC imports
from DIRAC.DataManagementSystem.Client.DataManager import DataManager
from DIRAC.Interfaces.API.Dirac import Dirac


@Script()
def main():
    if "JOBID" in os.environ:
        jobID = os.environ["JOBID"]
        dirac = Dirac()
        res = dirac.getJobJDL(jobID)
        if "InputData" in res["Value"]:
            lfn = res["Value"]["InputData"]

    dm = DataManager(["DIRACFileCatalog"])
    DIRAC.gLogger.notice(f"Check file: {lfn}")

    res = dm.getFile(lfn)

    if lfn in res["Value"]["Failed"]:
        if "Bad checksum" in res["Value"]["Failed"][lfn]:
            DIRAC.gLogger.notice(f"Bad checksum. Removing: {lfn}")
            res = dm.removeFile(lfn)
            if res["OK"]:
                DIRAC.gLogger.notice(f"Successfully removed: {lfn}")
            else:
                DIRAC.gLogger.error(res["Message"])
                DIRAC.exit(-1)
    else:
        DIRAC.gLogger.notice(f"Good checksum for: {lfn}")
        DIRAC.exit()


####################################################
if __name__ == "__main__":
    main()



"""
   Wrapper around the job class to build a workflow element (production step + job)
"""

__RCSID__ = "$Id$"

# generic imports
from copy import deepcopy
import json
from typing import Literal


# DIRAC imports
from CTADIRAC.Core.Utilities.tool_box import get_dataset_MQ

import DIRAC
from DIRAC.Interfaces.API.Job import Job
from DIRAC.ProductionSystem.Client.ProductionStep import ProductionStep
from DIRAC.Resources.Catalog.FileCatalogClient import FileCatalogClient
from CTADIRAC.Interfaces.API.CtapipeApplyModelsJob import CtapipeApplyModelsJob
from CTADIRAC.Interfaces.API.CtapipeMergeJob import CtapipeMergeJob
from CTADIRAC.Interfaces.API.CtapipeProcessJob import CtapipeProcessJob
from CTADIRAC.Interfaces.API.CtapipeTrainClassifierJob import CtapipeTrainClassifierJob
from CTADIRAC.Interfaces.API.CtapipeTrainEnergyJob import CtapipeTrainEnergyJob

from CTADIRAC.Interfaces.API.MCPipeJob import MCPipeJob


class WorkflowElementTypes:
    allowed_simulation_types: list[str] = ["mcsimulation"]
    allowed_processing_types: list[str] = [
        "ctapipeprocessing",
        "evndispprocessing",
        "merging",
        "ctapipetrainenergy",
        "ctapipetrainclassifier",
        "ctapipeapplymodels",
    ]
    we_types: list[str] = allowed_simulation_types + allowed_processing_types


class WorkflowElementDefinition:
    """Defines variables depending on WorkflowElement type"""

    def __init__(self, we_type: str) -> None:
        self.allowed_simulation_types: list[
            str
        ] = WorkflowElementTypes.allowed_simulation_types
        self.allowed_processing_types: list[
            str
        ] = WorkflowElementTypes.allowed_processing_types
        self.mandatory_keys: set[str] = {"MCCampaign", "configuration_id", "version"}
        self.constrained_input_keys: set[str] = {
            "pointing_dir",
            "zenith_angle",
            "moon",
            "sct",
            "div_ang",
        }
        self.constrained_job_keys: set[str] = {"catalogs", "moon", "group_size"}
        self.file_meta_fields: set[str] = {"nsb", "div_ang"}
        self.mandatory_job_config_keys: dict = {}
        self.we_type: Literal(WorkflowElementTypes.we_types) = we_type
        self.prod_step = ProductionStep()
        self.prod_step.Type = "DataReprocessing"
        self.initialize()

    def initialize(self) -> None:
        """Initialize variables depending on WE type"""
        if self.we_type == "mcsimulation":
            self.job = MCPipeJob()
            self.prod_step.Type = "MCSimulation"
            self.prod_step.Name = "MCSimulation"
            self.constrained_job_keys.update(
                [
                    "version",
                    "sct",
                    "particle",
                    "pointing_dir",
                    "layout",
                    "only_corsika",
                    "magic",
                    "div_ang",
                    "sequential",
                    "random_mono_probability",
                    "instrument_random_seeds",
                ]
            )
            self.constrained_job_keys.remove("group_size")

        elif self.we_type == "ctapipeprocessing":
            self.job = CtapipeProcessJob()
            self.prod_step.Name = "CtapipeProcessing"
            self.mandatory_job_config_keys = {"data_level", "output_extension"}
            self.file_meta_fields.add("split")

        elif self.we_type == "evndispprocessing":
            from CTADIRAC.Interfaces.API.EvnDispJob import EvnDispJob

            self.job = EvnDispJob()
            self.prod_step.Name = "EvnDisp"

        elif self.we_type == "merging":
            self.job = CtapipeMergeJob()
            self.prod_step.Type = "Merging"
            self.prod_step.Name = "Merging"
            self.mandatory_job_config_keys = {"output_extension"}
            self.file_meta_fields.add("split")

        elif self.we_type == "ctapipetrainenergy":
            self.job = CtapipeTrainEnergyJob()
            self.prod_step.Type = "Training"
            self.prod_step.Name = "CtapipeTrainEnergy"

        elif self.we_type == "ctapipetrainclassifier":
            self.job = CtapipeTrainClassifierJob()
            self.prod_step.Type = "Training"
            self.prod_step.Name = "CtapipeTrainClassifier"

        elif self.we_type == "ctapipeapplymodels":
            self.job = CtapipeApplyModelsJob()
            self.prod_step.Type = "ApplyModel"
            self.prod_step.Name = "CtapipeApplyModels"
            self.file_meta_fields.add("split")
        else:
            self.job = Job()
            self.mandatory_keys = {}


class WorkflowElement(WorkflowElementDefinition):
    """Composite class for workflow element (production step + job)"""

    def __init__(self, parent_prod_step: int, we_type: str) -> None:
        WorkflowElementDefinition.__init__(self, we_type)
        self.prod_step.ParentStep = parent_prod_step
        self.fc = FileCatalogClient()

    def set_constrained_job_attribute(self, key, value) -> None:
        """Set job attribute with constraints"""
        if self.we_type in self.allowed_simulation_types:
            self.set_constrained_mc_job_attribute(key, value)
        elif self.we_type in self.allowed_processing_types:
            self.set_constrained_processing_job_attribute(key, value)
        else:
            setattr(self.job, key, value)

    def set_constrained_mc_job_attribute(self, key, value) -> None:
        """Set simulation job attribute with constraints"""
        if key == "catalogs":
            # remove whitespaces between catalogs if there are some and separate between commas
            setattr(self.job, key, json.dumps(value.replace(", ", ",").split(sep=",")))
        elif key == "version":
            if "-sc" not in str(
                value
            ):  # if the version has not already been set by 'sct'
                self.job.version = str(value)
        elif key == "particle":
            self.job.set_particle(value)
        elif key == "pointing_dir":
            self.job.set_pointing_dir(value)
        elif key == "layout":
            self.job.set_layout(value)
        elif key == "sct":
            self.job.set_sct(value)
        elif key == "moon":
            self.job.set_moon(value.replace(", ", ",").split(sep=","))
        elif key == "only_corsika":
            self.job.set_only_corsika(value)
        elif key == "magic":
            self.job.set_magic(value)
        elif key == "sequential":
            self.job.set_sequential_mode(value)
        elif key == "div_ang":
            self.job.set_div_ang(value)
        elif key == "random_mono_probability":
            self.job.set_random_mono_probability(value)
        elif key == "instrument_random_seeds":
            self.job.set_instrument_random_seeds(value)

    def set_constrained_processing_job_attribute(self, key, value) -> None:
        """Set processing job attribute with constraints"""
        if key == "catalogs":
            # remove whitespaces between catalogs if there are some and separate between commas
            setattr(self.job, key, json.dumps(value.replace(", ", ",").split(sep=",")))
        elif key == "group_size":
            setattr(self.job, key, value)
            self.prod_step.GroupSize = self.job.group_size
        elif key == "moon":
            if value == "dark":
                self.job.output_file_metadata["nsb"] = 1
            elif value == "half":
                self.job.output_file_metadata["nsb"] = 5
            elif value == "full":
                self.job.output_file_metadata["nsb"] = 19

    def set_constrained_input_query(self, key, value) -> None:
        """Set input meta query with constraints"""
        if self.we_type in self.allowed_processing_types:
            self.set_constrained_processing_input_query(key, value)
        else:
            self.prod_step.Inputquery[key] = value

    def set_constrained_processing_input_query(self, key, value) -> None:
        """Set processing input meta query with constraints"""
        if key == "pointing_dir":
            if value == "North":
                self.prod_step.Inputquery["phiP"] = 180
            elif value == "South":
                self.prod_step.Inputquery["phiP"] = 0
        elif key == "zenith_angle":
            self.prod_step.Inputquery["thetaP"] = float(value)
        elif key == "sct":
            self.prod_step.Inputquery["sct"] = str(value)
        elif key == "moon":
            if value == "dark":
                self.prod_step.Inputquery["nsb"] = 1
            elif value == "half":
                self.prod_step.Inputquery["nsb"] = 5
            elif value == "full":
                self.prod_step.Inputquery["nsb"] = 19
        elif key == "div_ang":
            self.prod_step.Inputquery["div_ang"] = str(value)

    def build_job_attributes_from_job_config(self, workflow_step) -> None:
        """Build job attributes with job_config values"""
        for key in self.mandatory_job_config_keys:
            if key not in workflow_step["job_config"]:
                DIRAC.gLogger.error(f"{key} is mandatory")
                DIRAC.exit(-1)
        for key, value in workflow_step["job_config"].items():
            if value is not None:
                if key in self.constrained_job_keys:
                    self.set_constrained_job_attribute(key, value)
                elif key in self.file_meta_fields:
                    self.job.output_file_metadata[key] = value
                else:
                    setattr(self.job, key, value)
                    if key == "destination":
                        self.job.setDestination(value)
                    if key == "cpu_time":
                        self.job.setCPUTime(value)
                    if key == "tag":
                        self.job.setTag(value)
                    if key == "nb_processors":
                        self.job.setNumberOfProcessors(minNumberOfProcessors=value)

            elif key in self.mandatory_keys:
                DIRAC.gLogger.error(f"{key} is mandatory")
                DIRAC.exit(-1)
        for key in self.mandatory_keys:
            if key not in self.job.__dict__:
                DIRAC.gLogger.error(f"{key} is mandatory")
                DIRAC.exit(-1)

    def build_job_attributes_from_common(self, workflow_config) -> None:
        """Build job attributes concerning the whole production"""
        for key, value in workflow_config["Common"].items():
            if value is not None:
                setattr(self.job, key, value)
            elif key in self.mandatory_keys:
                DIRAC.gLogger.error(f"{key} is mandatory")
                DIRAC.exit(-1)

    def build_job_attributes_from_input(self) -> None:
        """Set job attributes from input meta query"""
        for key, value in self.prod_step.Inputquery.items():
            if key in self.file_meta_fields:
                try:
                    self.job.output_file_metadata[key] = value["="]
                except (KeyError, TypeError):
                    self.job.output_file_metadata[key] = value
                except Exception as e:
                    print(f"Unexpected exception for key {key}: {e}")
                    raise
            else:
                self.set_attribute(self.job, key, value, "=")

    def set_attribute(self, obj, key, value, sym) -> None:
        try:
            setattr(obj, key, value[f"{sym}"])
        except (KeyError, TypeError):
            setattr(obj, key, value)
        except Exception as e:
            print(f"Unexpected exception for key {key}: {e}")
            raise

    def build_job_attributes(self, workflow_config, workflow_step) -> None:
        """Set job attributes"""
        self.build_job_attributes_from_input()
        self.build_job_attributes_from_common(workflow_config)
        self.build_job_attributes_from_job_config(workflow_step)

    def build_input_data(self, workflow_step) -> None:
        """Build input data"""
        self.prod_step.Inputquery = {}
        if self.we_type not in self.allowed_simulation_types:
            if self.prod_step.ParentStep:
                self.prod_step.Inputquery = self.prod_step.ParentStep.Outputquery
            else:
                if workflow_step["input_meta_query"].get("dataset"):
                    self.prod_step.Inputquery = get_dataset_MQ(
                        workflow_step["input_meta_query"]["dataset"]
                    )

            self.refine_input_MQ(workflow_step)

    def refine_input_MQ(self, workflow_step) -> None:
        """refine input MQ by adding extra specification if needed"""
        for key, value in workflow_step["input_meta_query"].items():
            if value is not None:
                if key in self.constrained_input_keys:
                    self.set_constrained_input_query(key, value)
                elif (key != "dataset") and (key != "parentID"):
                    self.prod_step.Inputquery[key] = value

    def build_job_input_data(self, mode) -> None:
        """Limit the nb of input data to process (for testing purpose)"""
        if self.we_type not in self.allowed_simulation_types:
            res = self.fc.findFilesByMetadata(dict(self.prod_step.Inputquery))
            if not res["OK"]:
                DIRAC.gLogger.error(res["Message"])
                DIRAC.exit(-1)
            input_data_limit = self.job.input_limit
            input_data = res["Value"][:input_data_limit]
            if mode.lower() in ["wms", "local"]:
                self.job.setInputData(input_data)
            if not input_data and mode.lower() == "wms":
                DIRAC.gLogger.error("No job submitted: job must have input data")
                DIRAC.exit(-1)
            if mode.lower() == "ps" and self.job.input_limit:
                f = open("test_input_data.list", "w")
                for lfn in input_data:
                    f.write(lfn + "\n")
                f.close()
                DIRAC.gLogger.notice(
                    f"\t\tInput limit found: {len(input_data)} files dumped to test_input_data.list"
                )
                self.prod_step.Inputquery = {}

    def build_element_config(self, workflow_step) -> None:
        """Set job and production step attributes specific to the configuration"""
        if self.we_type in ["evndispprocessing", "merging"]:
            self.prod_step.GroupSize = self.job.group_size
        self.job.set_executable_sequence(debug=False)
        self.prod_step.Body = self.job.workflow.toXML()
        if "name" in workflow_step:
            self.prod_step.Name = workflow_step["name"]

    def build_job_output_data(self, workflow_step) -> None:
        """Build job output meta data"""
        if self.we_type in self.allowed_processing_types:
            metadata = deepcopy(self.prod_step.Inputquery)
            for key, value in workflow_step["job_config"].items():
                metadata[key] = value
            if self.we_type == "merging":
                merged = self.get_merging_level()
                metadata["merged"] = merged
            self.job.set_output_metadata(metadata)
        elif self.we_type in self.allowed_simulation_types:
            self.job.set_output_metadata()

    def get_merging_level(self) -> int:
        """Get the merging level from parent step or from the user query"""
        if self.prod_step.ParentStep:
            try:
                merged = self.prod_step.ParentStep.Outputquery["merged"]
            except KeyError:
                merged = 0
        else:
            merged = self.job.merged
        return merged

    def build_output_data(self) -> None:
        """Build output data from the job metadata and the metadata added on the files"""
        self.prod_step.Outputquery = deepcopy(self.job.output_metadata)
        for key, value in self.job.output_file_metadata.items():
            if isinstance(value, list):
                if len(value) > 1:
                    self.prod_step.Outputquery[key] = {"in": value}
                else:
                    self.prod_step.Outputquery[key] = value[0]
            else:
                self.prod_step.Outputquery[key] = value



from CTADIRAC.Interfaces.API.CTAJob import CTAJob


class CtapipeTrainClassifierJob(CTAJob):
    """Job extension class for ctapipe train particle classifier"""

    def __init__(self) -> None:
        super().__init__(we_type="ctapipetrainclassifier")
        self.setName("ctapipe_train-classifier")
        self.setType("Training")
        self.setNumberOfProcessors(minNumberOfProcessors=8)
        self.prog_name = "ctapipe-train-particle-classifier"
        # defaults
        self.data_level = 2
        self.data_output_pattern = "./Data/*.pkl"
        self.output_type = "Model"
        self.output_data_type = self.output_type

    def run_ctapipe_train_particle_classifier(self, i_step) -> None:
        proc_step = self.setExecutable(
            "./dirac_ctapipe-train-particle-classifier_wrapper",
            arguments=f"{self.options}",
            logFile="ctapipe_train-classifier_Log.txt",
        )
        proc_step["Value"]["name"] = f"Step{i_step}_ctapipe_train-classifier"
        proc_step["Value"]["descr_short"] = "Run ctapipe train classifier"

    def run_dedicated_software(self, i_step) -> None:
        self.run_ctapipe_train_particle_classifier(i_step)



"""
  Simple Wrapper on the Job class to handle EvnDisp Analysis
  for the Prod5b 2020 analysis from container
  https://forge.in2p3.fr/issues/42322
"""

__RCSID__ = "$Id$"

from CTADIRAC.Interfaces.API.CTAJob import CTAJob, MetadataDict


class EvnDispJob(CTAJob):
    def __init__(self) -> None:
        super().__init__(we_type="evndispsingjob")
        self.setName("Evndisplay_CalibReco")
        self.setType("EvnDispProcessing")
        self.package = "evndisplay"
        self.version = "eventdisplay-cta-dl1-prod5.v06"
        self.container = True
        self.program_category = "analysis"
        self.prog_name = "evndisp"
        self.configuration_id = 8
        self.output_data_level = 1
        self.base_path = "/vo.cta.in2p3.fr/MC/PROD5b/"
        self.ts_task_id = 0
        self.group_size = 1
        self.data_output_pattern = "./Data/*.simtel.DL1.tar.gz"

    def set_output_metadata(self, metadata: MetadataDict = {}) -> None:
        super().set_output_metadata(metadata)
        self.output_metadata["merged"] = 0

    def run_event_disp_software(self, i_step):
        ev_step = self.setExecutable(
            "./dirac_evndisp_wrapper",
            arguments=f"{self.options}",
            logFile="EvnDisp_Log.txt",
        )
        ev_step["Value"]["name"] = f"Step{i_step}_EvnDisplay"
        ev_step["Value"]["descr_short"] = "Run EvnDisplay"

    def run_dedicated_software(self, i_step) -> None:
        self.run_event_disp_software(i_step)



from CTADIRAC.Interfaces.API.CTAJob import MetadataDict


software_version = "v0.19.2"

SIMULATION_CONFIG = {
    "ID": 1,
    "input_meta_query": {"parentID": None, "dataset": None},
    "job_config": {
        "type": "MCSimulation",
        "version": "2020-06-29b",
        "array_layout": "Advanced-Baseline",
        "site": "LaPalma",
        "particle": "gamma-diffuse",
        "pointing_dir": "North",
        "zenith_angle": 20,
        "n_shower": 50,
        "magic": None,
        "sct": None,
        "moon": "dark",
        "start_run_number": 0,
    },
}
PROCESSING_CONFIG = {
    "ID": 2,
    "input_meta_query": {"parentID": 1, "moon": "dark"},
    "job_config": {
        "type": "CtapipeProcessing",
        "version": software_version,
        "output_extension": "DL2.h5",
        "options": "--config v3/dl0_to_dl2.yml --config v3/prod5b/subarray_north_alpha.yml",
        "array_layout": "Alpha",
        "data_level": 2,
        "group_size": 1,
    },
}
MERGING_CONFIG_1 = {
    "ID": 3,
    "input_meta_query": {"parentID": 2, "dataset": None},
    "job_config": {
        "type": "Merging",
        "version": software_version,
        "group_size": 5,
        "output_extension": "merged.DL2.h5",
    },
}
MERGING_CONFIG_2 = {
    "ID": 4,
    "input_meta_query": {"parentID": 3, "dataset": None},
    "job_config": {
        "type": "Merging",
        "version": software_version,
        "group_size": 2,
        "output_extension": "alpha_train_en_merged.DL2.h5",
        "options": "--no-dl1-images --no-true-images",
        "catalogs": "DIRACFileCatalog",
    },
}
COMMON_CONFIG = {
    "MCCampaign": "Prod5bTest",
    "configuration_id": 1,
    "base_path": "/vo.cta.in2p3.fr/tests/",
}

WORKFLOW_CONFIG = {
    "ProdSteps": [
        SIMULATION_CONFIG,
        PROCESSING_CONFIG,
        MERGING_CONFIG_1,
        MERGING_CONFIG_2,
    ],
    "Common": COMMON_CONFIG,
}

SIMULATION_OUTPUT_METADATA = MetadataDict(
    [
        ("array_layout", "Advanced-Baseline"),
        ("site", "LaPalma"),
        ("particle", "gamma-diffuse"),
        ("phiP", 180),
        ("thetaP", 20.0),
        ("sct", "False"),
        ("tel_sim_prog", "sim_telarray"),
        ("tel_sim_prog_version", "2020-06-29b"),
        ("data_level", -1),
        ("outputType", "Data"),
        ("configuration_id", 1),
        ("MCCampaign", "Prod5bTest"),
    ]
)

CTAPIPE_PROCESS_METADATA = MetadataDict(
    [
        ("array_layout", "Alpha"),
        ("site", "LaPalma"),
        ("particle", "gamma-diffuse"),
        ("phiP", 180),
        ("thetaP", 20.0),
        ("sct", "False"),
        ("tel_sim_prog", "sim_telarray"),
        ("tel_sim_prog_version", "2020-06-29b"),
        ("data_level", 2),
        ("outputType", "Data"),
        ("configuration_id", 1),
        ("MCCampaign", "Prod5bTest"),
        ("nsb", 1),
        ("type", "CtapipeProcessing"),
        ("version", software_version),
        ("output_extension", "DL2.h5"),
        (
            "options",
            "--config v3/dl0_to_dl2.yml --config v3/prod5b/subarray_north_alpha.yml",
        ),
        ("group_size", 1),
    ]
)

CTAPIPE_PROCESS_OUTPUT_METADATA = MetadataDict(
    [
        ("array_layout", "Alpha"),
        ("site", "LaPalma"),
        ("particle", "gamma-diffuse"),
        ("phiP", 180),
        ("thetaP", 20.0),
        ("sct", "False"),
        ("analysis_prog", "ctapipe-process"),
        ("analysis_prog_version", software_version),
        ("data_level", 2),
        ("outputType", "Data"),
        ("configuration_id", 1),
        ("MCCampaign", "Prod5bTest"),
    ]
)

MERGING1_METADATA = MetadataDict(
    [
        ("array_layout", "Alpha"),
        ("site", "LaPalma"),
        ("particle", "gamma-diffuse"),
        ("phiP", 180),
        ("thetaP", 20.0),
        ("sct", "False"),
        ("analysis_prog", "ctapipe-process"),
        ("analysis_prog_version", software_version),
        ("data_level", 2),
        ("outputType", "Data"),
        ("configuration_id", 1),
        ("MCCampaign", "Prod5bTest"),
        ("nsb", 1),
        ("type", "Merging"),
        ("version", software_version),
        ("group_size", 5),
        ("output_extension", "merged.DL2.h5"),
        ("merged", 0),
    ]
)
MERGING1_OUTPUT_METADATA = MetadataDict(
    [
        ("array_layout", "Alpha"),
        ("site", "LaPalma"),
        ("particle", "gamma-diffuse"),
        ("phiP", 180),
        ("thetaP", 20.0),
        ("sct", "False"),
        ("analysis_prog", "ctapipe-merge"),
        ("analysis_prog_version", software_version),
        ("data_level", 2),
        ("outputType", "Data"),
        ("configuration_id", 1),
        ("MCCampaign", "Prod5bTest"),
        ("merged", 1),
    ]
)



import json
import pytest
from CTADIRAC.Interfaces.API.MCPipeJob import MCPipeJob
from CTADIRAC.ProductionSystem.Client.Utilities.production_utils import (
    check_and_sort_workflow_config,
    check_destination_catalogs,
    check_id,
    check_parents,
    find_parent_prod_step,
    get_parents_list,
    instanciate_and_build_workflow_element,
    instantiate_workflow_element_from_type,
    sort_by_id,
)
from CTADIRAC.ProductionSystem.Client.WorkflowElement import WorkflowElement

from DIRAC.ProductionSystem.Client.ProductionStep import ProductionStep

from tests.unit.production import SIMULATION_CONFIG, WORKFLOW_CONFIG


def test_sort_by_id() -> None:
    output = sort_by_id([{"ID": 2}, {"ID": 1}])
    assert output == [{"ID": 1}, {"ID": 2}]


def test_check_id() -> None:
    with pytest.raises(SystemExit) as exc_info:
        check_id([{"ID": ""}])
    assert str(exc_info.value) == "-1"

    with pytest.raises(SystemExit) as exc_info:
        check_id([{"site": "Paranal"}])
    assert str(exc_info.value) == "-1"

    assert check_id([{"ID": 2}, {"ID": 1}]) is True


def test_check_parents() -> None:
    with pytest.raises(SystemExit) as exc_info:
        check_parents(
            [{"ID": 2, "input_meta_query": {"parentID": 3}, "job_config": {}}]
        )
    assert str(exc_info.value) == "-1"

    workflow_config = [
        {"ID": 1, "input_meta_query": {}, "job_config": {}},
        {"ID": 2, "input_meta_query": {"parentID": 1}, "job_config": {}},
    ]
    assert check_parents(workflow_config) is True


def test_instantiate_workflow_element_from_type() -> None:
    # Wrong WorkflowElement type:
    workflow_step = {
        "ID": 1,
        "input_meta_query": {},
        "job_config": {"type": "erroneous"},
    }
    with pytest.raises(SystemExit) as exc_info:
        instantiate_workflow_element_from_type(workflow_step, 1)
    assert str(exc_info.value) == "-1"

    # MCSimulation:
    workflow_step = {
        "ID": 1,
        "input_meta_query": {},
        "job_config": {"type": "MCSimulation"},
    }
    assert isinstance(
        instantiate_workflow_element_from_type(workflow_step, 1),
        WorkflowElement,
    )

    # CtapipeProcessing:
    workflow_step = {
        "ID": 1,
        "input_meta_query": {},
        "job_config": {"type": "CtapipeProcessing"},
    }
    assert isinstance(
        instantiate_workflow_element_from_type(workflow_step, 1),
        WorkflowElement,
    )

    # EvnDispProcessing:
    workflow_step = {
        "ID": 1,
        "input_meta_query": {},
        "job_config": {"type": "EvnDispProcessing"},
    }
    assert isinstance(
        instantiate_workflow_element_from_type(workflow_step, 1),
        WorkflowElement,
    )

    # Merging:
    workflow_step = {"ID": 1, "input_meta_query": {}, "job_config": {"type": "Merging"}}
    assert isinstance(
        instantiate_workflow_element_from_type(workflow_step, 1),
        WorkflowElement,
    )


def test_find_parent_prod_step() -> None:
    workflow_element_list: list[WorkflowElement] = [
        WorkflowElement(ProductionStep(), "merging"),
        WorkflowElement(ProductionStep(), "merging"),
    ]
    workflow_step = {"ID": 1, "input_meta_query": {}, "job_config": {"type": "Merging"}}
    assert find_parent_prod_step(workflow_element_list, workflow_step) is None

    workflow_step = {"ID": 1, "input_meta_query": {"parentID": None}, "job_config": {}}
    assert find_parent_prod_step(workflow_element_list, workflow_step) is None

    workflow_step = {"ID": 2, "input_meta_query": {"parentID": 1}, "job_config": {}}
    assert isinstance(
        find_parent_prod_step(workflow_element_list, workflow_step),
        ProductionStep,
    )


def test_get_parents_list() -> None:
    cases = [
        {
            "config": [
                {"ID": 1, "input_meta_query": {}, "job_config": {}},
                {"ID": 2, "input_meta_query": {"parentID": 1}, "job_config": {}},
            ],
            "expected_result": [1],
        },
        {
            "config": [
                {"ID": 1, "input_meta_query": {}, "job_config": {}},
                {"ID": 2, "input_meta_query": {"parentID": 1}, "job_config": {}},
                {"ID": 3, "input_meta_query": {"parentID": 1}, "job_config": {}},
            ],
            "expected_result": [1, 1],
        },
        {
            "config": [
                {"ID": 1, "input_meta_query": {}, "job_config": {}},
                {"ID": 2, "input_meta_query": {"parentID": 1}, "job_config": {}},
                {"ID": 3, "input_meta_query": {"parentID": 2}, "job_config": {}},
            ],
            "expected_result": [1, 2],
        },
    ]
    for case in cases:
        assert get_parents_list(case["config"]) == case["expected_result"]


def test_check_input_source_unicity() -> None:
    workflow_config = [
        {
            "ID": 1,
            "input_meta_query": {"parentID": 3, "dataset": "DATASET"},
            "job_config": {},
        }
    ]
    with pytest.raises(SystemExit) as exc_info:
        check_parents(workflow_config)
    assert str(exc_info.value) == "-1"

    workflow_config = [
        {
            "ID": 1,
            "input_meta_query": {"parentID": 1, "dataset": None},
            "job_config": {},
        },
    ]
    assert check_parents(workflow_config) is True

    workflow_config = [
        {
            "ID": 1,
            "input_meta_query": {"parentID": 1, "job_config": {}},
        },
    ]
    assert check_parents(workflow_config) is True


def test_check_destination_catalogs() -> None:
    default_catalogs = json.dumps(["DIRACFileCatalog", "TSCatalog"])
    workflow_element = WorkflowElement(ProductionStep(), "mcsimulation")
    workflow_element.job.catalogs = default_catalogs
    parents_list = [1]
    with pytest.raises(SystemExit) as exc_info:
        check_destination_catalogs(
            workflow_element,
            {
                "ID": 1,
                "input_meta_query": {},
                "job_config": {"catalogs": "DIRACFileCatalog"},
            },
            parents_list,
        )
    assert str(exc_info.value) == "-1"

    workflow_step = {
        "ID": 2,
        "input_meta_query": {},
        "job_config": {"catalogs": "DIRACFileCatalog"},
    }
    assert (
        check_destination_catalogs(workflow_element, workflow_step, parents_list)
        is True
    )

    workflow_step = {
        "ID": 1,
        "input_meta_query": {},
        "job_config": {"catalogs": "DIRACFileCatalog, TSCatalog"},
    }
    assert (
        check_destination_catalogs(workflow_element, workflow_step, parents_list)
        is True
    )
    workflow_step = {"ID": 1, "input_meta_query": {}, "job_config": {}}
    assert (
        check_destination_catalogs(workflow_element, workflow_step, parents_list)
        is True
    )


def test_check_and_sort_workflow_config() -> None:
    workflow_config = {
        "ProdSteps": [
            {
                "ID": 1,
                "input_meta_query": {"parentID": None, "dataset": None},
                "job_config": {},
            },
            {
                "ID": 2,
                "input_meta_query": {"parentID": 3, "moon": "dark"},
                "job_config": {},
            },
        ]
    }
    with pytest.raises(SystemExit) as exc_info:
        check_and_sort_workflow_config(workflow_config)
    assert str(exc_info.value) == "-1"


def test_instanciate_and_build_workflow_element() -> None:
    res_we = instanciate_and_build_workflow_element(
        SIMULATION_CONFIG, WORKFLOW_CONFIG, None, [1, 2, 3], "ps"
    )
    assert isinstance(res_we, WorkflowElement)
    assert isinstance(res_we.job, MCPipeJob)



#!/usr/bin/env python

"""
Run unittest test suite

Usage:
    python runTestSuite.py JobExecutionTests"""

import unittest
from DIRAC.Core.Base.Script import Script

# import test suite definition
from testSuiteDefinition import (
    JobExecutionTests,
    TransformationExecutionTests,
    ProductionExecutionTests,
    ProductionConfigurationTests,
    ProdSystemFullTests,
    ClientDMSTests,
    ClientDIRACFileCatalogTests,
    ClientRucioFileCatalogTests,
    ClientDMSDatasetTests,
)

test_cases = [
    "JobExecutionTests TransformationExecutionTests ProductionExecutionTests "
    "ProductionConfigurationTests ProdSystemFullTests ClientDMSTests ClientDIRACFileCatalogTests "
    "ClientRucioFileCatalogTests ClientDMSDatasetTests "
]

Script.registerArgument(description=["tests: Test cases list"], values=test_cases)
Script.parseCommandLine()

from DIRAC import gLogger

argss = Script.getPositionalArgs()
if len(argss) < 1:
    Script.showHelp()

debug = Script.localCfg.getDebugMode()

if debug > 0:
    log_file = "TestJobs.log"
    gLogger.registerBackend("file", {"FileName": f"{log_file}"})
    gLogger.setLevel(debug)


suite = unittest.TestSuite()

for case in argss:
    test = unittest.defaultTestLoader.loadTestsFromTestCase(eval(case))
    suite.addTest(test)

testResult = unittest.TextTestRunner(verbosity=2).run(suite)

if not testResult.wasSuccessful():
    exit(1)



""" DISET request handler base class for the ProvenanceDB
"""

__RCSID__ = "$Id$"
# imports
import json

# from DIRAC
from DIRAC import gLogger, S_OK, S_ERROR
from DIRAC.Core.DISET.RequestHandler import RequestHandler

# from CTADIRAC
from CTADIRAC.DataManagementSystem.DB.ProvenanceDB import ProvenanceDB


class ProvenanceManagerHandler(RequestHandler):

    """
    .. class:: ProvenanceManagerHandler
    ProvenanceDB interface in the DISET framework.
    """

    __provenanceDB = None

    @classmethod
    def initializeHandler(cls, serviceInfoDict):
        """initialize handler"""

        try:
            cls.__provenanceDB = ProvenanceDB()
        except RuntimeError as error:
            gLogger.exception(error)
            return S_ERROR(error)

        return S_OK()

    def _parseRes(cls, res):
        if not res["OK"]:
            gLogger.error("ProvenanceManager failure", res["Message"])
        return res

    types_addActivityDescription = [(str,)]

    def export_addActivityDescription(cls, rowJSON):
        """
        #Insert ActivityDescription
        #:param rowSON
        #:return:
        """

        rowDict = json.loads(rowJSON)
        res = cls.__provenanceDB.addActivityDescription(rowDict)
        return cls._parseRes(res)

    types_addActivity = [(str,)]

    def export_addActivity(cls, rowJSON):
        """
        #Insert ActivityDescription
        #:param rowJSON
        #:return:
        """

        rowDict = json.loads(rowJSON)
        res = cls.__provenanceDB.addActivity(rowDict)
        return cls._parseRes(res)

    types_addWasAssociatedWith = [(str,)]

    def export_addWasAssociatedWith(cls, rowJSON):
        """
        #Insert WasAssociatedWith
        #:param rowJSON
        #:return:
        """

        rowDict = json.loads(rowJSON)
        res = cls.__provenanceDB.addWasAssociatedWith(rowDict)
        return cls._parseRes(res)

    types_addAgent = [(str,)]

    def export_addAgent(cls, rowJSON):
        """
        Insert Agent
        :param rowJSON
        :return:
        """

        rowDict = json.loads(rowJSON)
        res = cls.__provenanceDB.addAgent(rowDict)
        return cls._parseRes(res)

    types_addDatasetDescription = [(str,)]

    def export_addDatasetDescription(cls, rowJSON):
        """
        Insert DatasetDescription
        :param rowJSON
        :return:
        """

        rowDict = json.loads(rowJSON)
        res = cls.__provenanceDB.addDatasetDescription(rowDict)
        return cls._parseRes(res)

    types_addUsageDescription = [(str,)]

    def export_addUsageDescription(cls, rowJSON):
        """
        Insert UsageDescription
        :param rowJSON
        :return:
        """

        rowDict = json.loads(rowJSON)
        res = cls.__provenanceDB.addUsageDescription(rowDict)
        return cls._parseRes(res)

    types_addGenerationDescription = [(str,)]

    def export_addGenerationDescription(cls, rowJSON):
        """
        Insert UsageDescription
        :param rowJSON
        :return:
        """

        rowDict = json.loads(rowJSON)
        res = cls.__provenanceDB.addGenerationDescription(rowDict)
        return cls._parseRes(res)

    types_addValueDescription = [(str,)]

    def export_addValueDescription(cls, rowJSON):
        """
        Insert UsageDescription
        :param rowJSON
        :return:
        """

        rowDict = json.loads(rowJSON)
        res = cls.__provenanceDB.addValueDescription(rowDict)
        return cls._parseRes(res)

    types_addDatasetEntity = [(str,)]

    def export_addDatasetEntity(cls, rowJSON):
        """
        Insert DatasetEntity
        :param rowJSON
        :return:
        """

        rowDict = json.loads(rowJSON)
        res = cls.__provenanceDB.addDatasetEntity(rowDict)
        return cls._parseRes(res)

    types_addWasAttributedTo = [(str,)]

    def export_addWasAttributedTo(cls, rowJSON):
        """
        Insert WasAttributedTo
        :param rowJSON
        :return:
        """

        rowDict = json.loads(rowJSON)
        res = cls.__provenanceDB.addWasAttributedTo(rowDict)
        return cls._parseRes(res)

    types_addUsed = [(str,)]

    def export_addUsed(cls, rowJSON):
        """
        Insert Used
        :param rowJSON
        :return:
        """

        rowDict = json.loads(rowJSON)
        res = cls.__provenanceDB.addUsed(rowDict)
        return cls._parseRes(res)

    types_addWasGeneratedBy = [(str,)]

    def export_addWasGeneratedBy(cls, rowJSON):
        """
        Insert WasGeneratedBy
        :param rowJSON
        :return:
        """

        rowDict = json.loads(rowJSON)
        res = cls.__provenanceDB.addWasGeneratedBy(rowDict)
        return cls._parseRes(res)

    types_addValueEntity = [(str,)]

    def export_addValueEntity(cls, rowJSON):
        """
        Insert ValueEntity
        :param rowJSON
        :return:
        """

        rowDict = json.loads(rowJSON)
        res = cls.__provenanceDB.addValueEntity(rowDict)
        return cls._parseRes(res)

    types_addWasConfiguredBy = [(str,)]

    def export_addWasConfiguredBy(cls, rowJSON):
        """
        Insert WasConfiguredBy
        :param rowJSON
        :return:
        """

        rowDict = json.loads(rowJSON)
        res = cls.__provenanceDB.addWasConfiguredBy(rowDict)
        return cls._parseRes(res)

    types_addParameter = [(str,)]

    def export_addParameter(cls, rowJSON):
        """
        Insert Parameter
        :param rowJSON
        :return:
        """

        rowDict = json.loads(rowJSON)
        res = cls.__provenanceDB.addParameter(rowDict)
        return cls._parseRes(res)

    types_addConfigFile = [(str,)]

    def export_addConfigFile(cls, rowJSON):
        """
        Insert ConfigFile
        :param rowJSON
        :return:
        """

        rowDict = json.loads(rowJSON)
        res = cls.__provenanceDB.addConfigFile(rowDict)
        return cls._parseRes(res)

    types_addParameterDescription = [(str,)]

    def export_addParameterDescription(cls, rowJSON):
        """
        Insert ParameterDescription
        :param rowJSON
        :return:
        """

        rowDict = json.loads(rowJSON)
        res = cls.__provenanceDB.addParameterDescription(rowDict)
        return cls._parseRes(res)

    types_addConfigFileDescription = [(str,)]

    def export_addConfigFileDescription(cls, rowJSON):
        """
        Insert ConfigFileDescription
        :param rowJSON
        :return:
        """

        rowDict = json.loads(rowJSON)
        res = cls.__provenanceDB.addConfigFileDescription(rowDict)
        return cls._parseRes(res)

    types_getAgents = []

    def export_getAgents(cls):
        """
        Get Agents
        :return:
        """

        res = cls.__provenanceDB.getAgents()
        return cls._parseRes(res)

    types_getAgentKey = []

    def export_getAgentKey(cls, agent_id):
        """
        Get Agent Key
        :return:
        """

        res = cls.__provenanceDB.getAgentKey(agent_id)
        return cls._parseRes(res)

    types_getDatasetEntity = [(str,)]

    def export_getDatasetEntity(cls, guid):
        """
        Get DatasetEntity
        :param datasetEntity guid
        :return:
        """

        res = cls.__provenanceDB.getDatasetEntity(guid)
        return cls._parseRes(res)

    types_updateDatasetEntity = [(str,)]

    def export_updateDatasetEntity(cls, guid, invalidatedAtTime):
        """
        Update DatasetEntity
        :param datasetEntity guid invalidatedAtTime
        :return:
        """

        res = cls.__provenanceDB.updateDatasetEntity(guid, invalidatedAtTime)
        return cls._parseRes(res)

    types_getUsageDescription = []

    def export_getUsageDescription(cls, activityDescription_id, role):
        """
        Get UsageDescription
        :param activity_id, role
        :return {'id':usageDescription_id, 'entityDescription_id':entityDescription_id}
        """

        res = cls.__provenanceDB.getUsageDescription(activityDescription_id, role)
        return cls._parseRes(res)

    types_getGenerationDescription = []

    def export_getGenerationDescription(cls, activityDescription_id, role):
        """
        Get GenerationDescription
        :param activity_id, role
        :return {'id':GenerationDescription_id, 'entityDescription_id':entityDescription_id}
        """

        res = cls.__provenanceDB.getGenerationDescription(activityDescription_id, role)
        return cls._parseRes(res)

    types_getParameterDescription = []

    def export_getParameterDescription(cls, activityDescription_id, parameter_name):
        """
        Get ParameterDescription
        :param activity_id, parameter_name
        :return {'id':ParameterDescription_id}
        """

        res = cls.__provenanceDB.getParameterDescription(
            activityDescription_id, parameter_name
        )
        return cls._parseRes(res)

    types_getConfigFileDescription = []

    def export_getConfigFileDescription(cls, activityDescription_id, configFile_name):
        """
        Get ConfigFileDescription
        :param activity_id, configFile
        :return {'id':ConfigFileDescription_id}
        """

        res = cls.__provenanceDB.getConfigFileDescription(
            activityDescription_id, configFile_name
        )
        return cls._parseRes(res)

    types_getActivityDescriptionKey = []

    def export_getActivityDescriptionKey(
        cls, activityDescription_name, activityDescription_version
    ):
        """
        Get ActivityDescriptionKey
        :param activityDescription_name, activityDescription_version
        :return {'internal_key':activityDescription_key}
        """

        res = cls.__provenanceDB.getActivityDescriptionKey(
            activityDescription_name, activityDescription_version
        )
        return cls._parseRes(res)
